# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xdi',
 'xdi._common',
 'xdi._common.asyncio',
 'xdi._common.datapath',
 'xdi._common.imports',
 'xdi.drivers',
 'xdi.drivers.sanic',
 'xdi.providers']

package_data = \
{'': ['*']}

install_requires = \
['typing-extensions>=4.1.1,<5.0.0']

setup_kwargs = {
    'name': 'xdi',
    'version': '0.1.0',
    'description': 'Dependency injection library',
    'long_description': '# XDI\n\nA super fast dependency injection library for python.\n\n\n\n## Why use dependency injection?\n\nTake a look at the following code.\n\n```python\nimport os\n\nclass ApiClient:\n\n    def __init__(self):\n        self.api_url = os.getenv("API_URL")  # a dependency\n        self.api_key = os.getenv("API_KEY")  # a dependency\n\n    def get(self, path: str, *, token: str):\n        return { url: f\'{self.api_url}/{path}\', status: \'ok\', data: [] }\n\n\n\nclass Service:\n\n    def __init__(self):\n        self._api_client = ApiClient()  # a dependency\n        # do some init\n\n    def do_something(self):\n        res = self._api_client.get(\'abc\')\n        print("Service doing something")\n\n\ndef some_func() -> None:\n    service = Service()  # a dependency\n    service.do_something()\n    print("serivce has done something")\n\n\nif __name__ == "__main__":\n    some_func()\n\n```\n\nThis code will run as expected. However:-\n\n1. **Testing it will be difficult**\n    \n    For example, to test `Service` we need a fake `ApiClient` as we don\'t what to \n    make real api calls or might not have the credentials to do so. \n    Since `Service` creates it\'s own `ApiClient` instance, it is impossible to \n    safely mock the `ApiClient` for tests. \n\n2. **Lacks flexibility and extensibility**\n\n    It\'s imppossible to create an additional `ApiClient` instance that uses a \n    different `API_URL` and/or `API_KEY`.\n\n\n\nSo what do we do? We should decouple our objects from their dependencies. \n\nThat is, objects should not create each other anymore. They should provide a way \nto inject the dependencies instead.\n\n\nHere\'s how.\n\n```python\n\nclass ApiClient:\n\n    def __init__(self, api_url: str, api_key: str): # we let the caller provide the dependencies\n        self.api_url = api_url  \n        self.api_key = api_key\n\n\nclass Service:\n\n    def __init__(self, api_client: ApiClient): # we let the caller provide the dependency\n        self._api_client = api_client \n\n\ndef some_func(service: Service): # we let the caller provide the dependency\n    service.do_something()\n    print("serivce has done something")\n\n```\n\nCongratulations, your code is now loosely coupled. \n\nBut remember, with freedom comes more responsibility.\n\nThe responsibility is left to the "caller" who has to know, assemble and provide the dependencies.\n\n```python\nsome_func(\n    service=Service(\n        api_client=ApiClient(\n            api_key=os.getenv("API_KEY"),\n            api_url=os.getenv("TIMEOUT"),\n        ),\n    ),\n)\n\n```\nThis quickly becomes a problem when you what to use `some_func()` from multiple places.\nDuplicating the assembly code with make it harder to change in the future.\n\n\n\n\n### With XDI.\n\n\n```python\nfrom xdi import Injector, inject, context\n\n@inject  # tell the di to inject dependencies\ndef some_func(service: Service):\n    service.do_something()\n\n\ninjector = Injector()\n\n# register the ApiClient\ninjector.factory(\n    ApiClient, \n    api_url=os.getenv("API_URL"), # <-- provide value from env\n    api_key=os.getenv(\'API_KEY\')  # <-- provide value from env\n).singleton()  # <-- make it provide only one instance.\n\n\n# Register the Service. \ninjector.factory(Service) \n# Since we did not specify a value for api_client. `ApiClient` will get injected \n# as it matches the type annotation `api_client: ApiClient`\n\n\n# Run the injector\ninjector.run_forever()\n\nsome_func() # <-- dependency `Service` is injected automatically\n\n```\n\n\n\n## Installation\n\nInstall from [PyPi](https://pypi.org/project/xdi/)\n\n```\npip install xdi\n```\n\n\n## Documentation\n\nComing soon.\n\n\n## Production\n\n__This package is still in active development and should not be used in production environment__\n\n',
    'author': 'David Kyalo',
    'author_email': 'kyalo@qwertie.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/davidkyalo/xdi',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
